@@ -5,6 +5,7 @@ import (
 	"testing"
 
 	"github.com/docker/cli/internal/test"
+	"github.com/moby/moby/client"
 	"gotest.tools/v3/assert"
 	is "gotest.tools/v3/assert/cmp"
 )
@@ -17,7 +18,7 @@ func TestCliNewTagCommandErrors(t *testing.T) {
 	}
 	expectedError := "'tag' requires 2 arguments"
 	for _, args := range testCases {
-		cmd := NewTagCommand(test.NewFakeCli(&fakeClient{}))
+		cmd := newTagCommand(test.NewFakeCli(&fakeClient{}))
 		cmd.SetArgs(args)
 		cmd.SetOut(io.Discard)
 		cmd.SetErr(io.Discard)
@@ -26,12 +27,12 @@ func TestCliNewTagCommandErrors(t *testing.T) {
 }
 
 func TestCliNewTagCommand(t *testing.T) {
-	cmd := NewTagCommand(
+	cmd := newTagCommand(
 		test.NewFakeCli(&fakeClient{
-			imageTagFunc: func(image string, ref string) error {
-				assert.Check(t, is.Equal("image1", image))
-				assert.Check(t, is.Equal("image2", ref))
-				return nil
+			imageTagFunc: func(options client.ImageTagOptions) (client.ImageTagResult, error) {
+				assert.Check(t, is.Equal("image1", options.Source))
+				assert.Check(t, is.Equal("image2", options.Target))
+				return client.ImageTagResult{}, nil
 			},
 		}))
 	cmd.SetArgs([]string{"image1", "image2"})
