@@ -1,92 +1,94 @@
-package client // import "github.com/docker/docker/client"
+package client
 
 import (
-	"bytes"
-	"context"
 	"fmt"
-	"io"
+	"math/rand"
 	"net/http"
-	"strings"
 	"testing"
 
 	cerrdefs "github.com/containerd/errdefs"
-	"github.com/docker/docker/testutil"
 	"gotest.tools/v3/assert"
 	is "gotest.tools/v3/assert/cmp"
 )
 
 func TestImageTagError(t *testing.T) {
-	client := &Client{
-		client: newMockClient(errorMock(http.StatusInternalServerError, "Server error")),
-	}
+	client, err := New(WithMockClient(errorMock(http.StatusInternalServerError, "Server error")))
+	assert.NilError(t, err)
 
-	err := client.ImageTag(context.Background(), "image_id", "repo:tag")
+	_, err = client.ImageTag(t.Context(), ImageTagOptions{Source: "image_id", Target: "repo:tag"})
 	assert.Check(t, is.ErrorType(err, cerrdefs.IsInternal))
 }
 
 // Note: this is not testing all the InvalidReference as it's the responsibility
 // of distribution/reference package.
 func TestImageTagInvalidReference(t *testing.T) {
-	client := &Client{
-		client: newMockClient(errorMock(http.StatusInternalServerError, "Server error")),
-	}
+	client, err := New(WithMockClient(errorMock(http.StatusInternalServerError, "Server error")))
+	assert.NilError(t, err)
 
-	err := client.ImageTag(context.Background(), "image_id", "aa/asdf$$^/aa")
-	assert.Check(t, is.Error(err, `Error parsing reference: "aa/asdf$$^/aa" is not a valid repository/tag: invalid reference format`))
+	_, err = client.ImageTag(t.Context(), ImageTagOptions{Source: "image_id", Target: "aa/asdf$$^/aa"})
+	assert.Check(t, is.Error(err, `error parsing reference: "aa/asdf$$^/aa" is not a valid repository/tag: invalid reference format`))
 }
 
 // Ensure we don't allow the use of invalid repository names or tags; these tag operations should fail.
 func TestImageTagInvalidSourceImageName(t *testing.T) {
-	ctx := context.Background()
+	ctx := t.Context()
 
-	client := &Client{
-		client: newMockClient(errorMock(http.StatusInternalServerError, "client should not have made an API call")),
-	}
+	client, err := New(WithMockClient(errorMock(http.StatusInternalServerError, "client should not have made an API call")))
+	assert.NilError(t, err)
 
 	invalidRepos := []string{"fo$z$", "Foo@3cc", "Foo$3", "Foo*3", "Fo^3", "Foo!3", "F)xcz(", "fo%asd", "aa/asdf$$^/aa"}
 	for _, repo := range invalidRepos {
 		t.Run("invalidRepo/"+repo, func(t *testing.T) {
 			t.Parallel()
-			err := client.ImageTag(ctx, "busybox", repo)
+			_, err := client.ImageTag(ctx, ImageTagOptions{Source: "busybox", Target: repo})
 			assert.Check(t, is.ErrorContains(err, "not a valid repository/tag"))
 		})
 	}
 
-	longTag := testutil.GenerateRandomAlphaOnlyString(121)
+	longTag := generateRandomAlphaOnlyString(121)
 	invalidTags := []string{"repo:fo$z$", "repo:Foo@3cc", "repo:Foo$3", "repo:Foo*3", "repo:Fo^3", "repo:Foo!3", "repo:%goodbye", "repo:#hashtagit", "repo:F)xcz(", "repo:-foo", "repo:..", longTag}
 	for _, repotag := range invalidTags {
 		t.Run("invalidTag/"+repotag, func(t *testing.T) {
 			t.Parallel()
-			err := client.ImageTag(ctx, "busybox", repotag)
+			_, err := client.ImageTag(ctx, ImageTagOptions{Source: "busybox", Target: repotag})
 			assert.Check(t, is.ErrorContains(err, "not a valid repository/tag"))
 		})
 	}
 
 	t.Run("test repository name begin with '-'", func(t *testing.T) {
 		t.Parallel()
-		err := client.ImageTag(ctx, "busybox:latest", "-busybox:test")
-		assert.Check(t, is.ErrorContains(err, "Error parsing reference"))
+		_, err := client.ImageTag(ctx, ImageTagOptions{Source: "busybox:latest", Target: "-busybox:test"})
+		assert.Check(t, is.ErrorContains(err, "error parsing reference"))
 	})
 
 	t.Run("test namespace name begin with '-'", func(t *testing.T) {
 		t.Parallel()
-		err := client.ImageTag(ctx, "busybox:latest", "-test/busybox:test")
-		assert.Check(t, is.ErrorContains(err, "Error parsing reference"))
+		_, err := client.ImageTag(ctx, ImageTagOptions{Source: "busybox:latest", Target: "-test/busybox:test"})
+		assert.Check(t, is.ErrorContains(err, "error parsing reference"))
 	})
 
 	t.Run("test index name begin with '-'", func(t *testing.T) {
 		t.Parallel()
-		err := client.ImageTag(ctx, "busybox:latest", "-index:5000/busybox:test")
-		assert.Check(t, is.ErrorContains(err, "Error parsing reference"))
+		_, err := client.ImageTag(ctx, ImageTagOptions{Source: "busybox:latest", Target: "-index:5000/busybox:test"})
+		assert.Check(t, is.ErrorContains(err, "error parsing reference"))
 	})
 }
 
-func TestImageTagHexSource(t *testing.T) {
-	client := &Client{
-		client: newMockClient(errorMock(http.StatusOK, "OK")),
+func generateRandomAlphaOnlyString(n int) string {
+	// make a really long string
+	letters := []byte("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ")
+	b := make([]byte, n)
+	for i := range b {
+		b[i] = letters[rand.Intn(len(letters))] //nolint: gosec // G404: Use of weak random number generator (math/rand instead of crypto/rand)
 	}
+	return string(b)
+}
 
-	err := client.ImageTag(context.Background(), "0d409d33b27e47423b049f7f863faa08655a8c901749c2b25b93ca67d01a470d", "repo:tag")
+func TestImageTagHexSource(t *testing.T) {
+	client, err := New(WithMockClient(mockResponse(http.StatusOK, nil, "OK")))
+	assert.NilError(t, err)
+
+	_, err = client.ImageTag(t.Context(), ImageTagOptions{Source: "0d409d33b27e47423b049f7f863faa08655a8c901749c2b25b93ca67d01a470d", Target: "repo:tag"})
 	assert.NilError(t, err)
 }
 
@@ -147,28 +149,21 @@ func TestImageTag(t *testing.T) {
 		},
 	}
 	for _, tagCase := range tagCases {
-		client := &Client{
-			client: newMockClient(func(req *http.Request) (*http.Response, error) {
-				if !strings.HasPrefix(req.URL.Path, expectedURL) {
-					return nil, fmt.Errorf("expected URL '%s', got '%s'", expectedURL, req.URL)
+		client, err := New(WithMockClient(func(req *http.Request) (*http.Response, error) {
+			if err := assertRequest(req, http.MethodPost, expectedURL); err != nil {
+				return nil, err
+			}
+			query := req.URL.Query()
+			for key, expected := range tagCase.expectedQueryParams {
+				actual := query.Get(key)
+				if actual != expected {
+					return nil, fmt.Errorf("%s not set in URL query properly. Expected '%s', got %s", key, expected, actual)
 				}
-				if req.Method != http.MethodPost {
-					return nil, fmt.Errorf("expected POST method, got %s", req.Method)
-				}
-				query := req.URL.Query()
-				for key, expected := range tagCase.expectedQueryParams {
-					actual := query.Get(key)
-					if actual != expected {
-						return nil, fmt.Errorf("%s not set in URL query properly. Expected '%s', got %s", key, expected, actual)
-					}
-				}
-				return &http.Response{
-					StatusCode: http.StatusOK,
-					Body:       io.NopCloser(bytes.NewReader([]byte(""))),
-				}, nil
-			}),
-		}
-		err := client.ImageTag(context.Background(), "image_id", tagCase.reference)
+			}
+			return mockResponse(http.StatusOK, nil, "")(req)
+		}))
+		assert.NilError(t, err)
+		_, err = client.ImageTag(t.Context(), ImageTagOptions{Source: "image_id", Target: tagCase.reference})
 		assert.NilError(t, err)
 	}
 }
