@@ -1,11 +1,11 @@
-package container // import "github.com/docker/docker/daemon/cluster/executor/container"
+package container
 
 import (
 	"context"
 	"testing"
 	"time"
 
-	"github.com/docker/docker/daemon"
+	"github.com/moby/moby/v2/daemon"
 	"github.com/moby/swarmkit/v2/api"
 )
 
