@@ -3,17 +3,17 @@ package containerd
 import (
 	"context"
 
-	"github.com/containerd/containerd/images"
-	"github.com/docker/docker/api/types/backend"
-	"github.com/docker/docker/api/types/events"
+	c8dimages "github.com/containerd/containerd/v2/core/images"
+	"github.com/moby/moby/api/types/events"
+	"github.com/moby/moby/v2/daemon/server/imagebackend"
 )
 
 // LogImageEvent generates an event related to an image with only the default attributes.
-func (i *ImageService) LogImageEvent(imageID, refName string, action events.Action) {
-	ctx := context.TODO()
+func (i *ImageService) LogImageEvent(ctx context.Context, imageID, refName string, action events.Action) {
+	ctx = context.WithoutCancel(ctx)
 	attributes := map[string]string{}
 
-	img, err := i.GetImage(ctx, imageID, backend.GetImageOpts{})
+	img, err := i.GetImage(ctx, imageID, imagebackend.GetImageOpts{})
 	if err == nil && img.Config != nil {
 		// image has not been removed yet.
 		// it could be missing if the event is `delete`.
@@ -29,7 +29,7 @@ func (i *ImageService) LogImageEvent(imageID, refName string, action events.Acti
 }
 
 // logImageEvent generates an event related to an image with only name attribute.
-func (i *ImageService) logImageEvent(img images.Image, refName string, action events.Action) {
+func (i *ImageService) logImageEvent(img c8dimages.Image, refName string, action events.Action) {
 	attributes := map[string]string{}
 	if refName != "" {
 		attributes["name"] = refName
