@@ -1,10 +1,11 @@
-package images // import "github.com/docker/docker/daemon/images"
+package images
 
 import (
 	"context"
 
-	"github.com/docker/distribution/reference"
-	"github.com/docker/docker/image"
+	"github.com/distribution/reference"
+	"github.com/moby/moby/api/types/events"
+	"github.com/moby/moby/v2/daemon/internal/image"
 )
 
 // TagImage adds the given reference to the image ID provided.
@@ -16,6 +17,6 @@ func (i *ImageService) TagImage(ctx context.Context, imageID image.ID, newTag re
 	if err := i.imageStore.SetLastUpdated(imageID); err != nil {
 		return err
 	}
-	i.LogImageEvent(imageID.String(), reference.FamiliarString(newTag), "tag")
+	i.LogImageEvent(ctx, imageID.String(), reference.FamiliarString(newTag), events.ActionTag)
 	return nil
 }
