@@ -1,9 +1,9 @@
-package daemon // import "github.com/docker/docker/daemon"
+package daemon
 
 import (
 	"strings"
 
-	"github.com/docker/docker/container"
+	"github.com/moby/moby/v2/daemon/container"
 )
 
 // excludeByIsolation is a platform specific helper function to support PS
