@@ -1,6 +1,6 @@
 //go:build !linux && !freebsd
 
-package logger // import "github.com/docker/docker/daemon/logger"
+package logger
 
 import (
 	"errors"
