@@ -9,14 +9,13 @@ import (
 	"strings"
 	"testing"
 
-	"github.com/docker/distribution"
-	"github.com/docker/distribution/manifest"
-	"github.com/docker/distribution/manifest/manifestlist"
-	"github.com/docker/distribution/manifest/schema2"
-	"github.com/docker/docker/integration-cli/cli"
-	"github.com/docker/docker/integration-cli/cli/build"
+	"github.com/moby/moby/v2/integration-cli/cli"
+	"github.com/moby/moby/v2/integration-cli/cli/build"
 	"github.com/opencontainers/go-digest"
+	"github.com/opencontainers/image-spec/specs-go"
+	ocispec "github.com/opencontainers/image-spec/specs-go/v1"
 	"gotest.tools/v3/assert"
+	is "gotest.tools/v3/assert/cmp"
 	"gotest.tools/v3/icmd"
 	"gotest.tools/v3/skip"
 )
@@ -25,7 +24,7 @@ import (
 // tags for the same image) are not also pulled down.
 //
 // Ref: docker/docker#8141
-func testPullImageWithAliases(c *testing.T) {
+func (s *DockerRegistrySuite) TestPullImageWithAliases(t *testing.T) {
 	const imgRepo = privateRegistryURL + "/dockercli/busybox"
 
 	var repos []string
@@ -35,52 +34,44 @@ func testPullImageWithAliases(c *testing.T) {
 
 	// Tag and push the same image multiple times.
 	for _, repo := range repos {
-		cli.DockerCmd(c, "tag", "busybox", repo)
-		cli.DockerCmd(c, "push", repo)
+		cli.DockerCmd(t, "tag", "busybox", repo)
+		cli.DockerCmd(t, "push", repo)
 	}
 
 	// Clear local images store.
 	args := append([]string{"rmi"}, repos...)
-	cli.DockerCmd(c, args...)
+	cli.DockerCmd(t, args...)
 
 	// Pull a single tag and verify it doesn't bring down all aliases.
-	cli.DockerCmd(c, "pull", repos[0])
-	cli.DockerCmd(c, "inspect", repos[0])
+	cli.DockerCmd(t, "pull", repos[0])
+	cli.DockerCmd(t, "inspect", repos[0])
 	for _, repo := range repos[1:] {
 		_, _, err := dockerCmdWithError("inspect", repo)
-		assert.ErrorContains(c, err, "", "Image %v shouldn't have been pulled down", repo)
+		assert.ErrorContains(t, err, "", "Image %v shouldn't have been pulled down", repo)
 	}
 }
 
-func (s *DockerRegistrySuite) TestPullImageWithAliases(c *testing.T) {
-	testPullImageWithAliases(c)
-}
-
-func (s *DockerSchema1RegistrySuite) TestPullImageWithAliases(c *testing.T) {
-	testPullImageWithAliases(c)
-}
-
-// testConcurrentPullWholeRepo pulls the same repo concurrently.
-func testConcurrentPullWholeRepo(c *testing.T) {
+// TestConcurrentPullWholeRepo pulls the same repo concurrently.
+func (s *DockerRegistrySuite) TestConcurrentPullWholeRepo(t *testing.T) {
 	const imgRepo = privateRegistryURL + "/dockercli/busybox"
 
 	var repos []string
 	for _, tag := range []string{"recent", "fresh", "todays"} {
 		repo := fmt.Sprintf("%v:%v", imgRepo, tag)
-		buildImageSuccessfully(c, repo, build.WithDockerfile(fmt.Sprintf(`
+		cli.BuildCmd(t, repo, build.WithDockerfile(fmt.Sprintf(`
 		    FROM busybox
 		    ENTRYPOINT ["/bin/echo"]
 		    ENV FOO foo
 		    ENV BAR bar
 		    CMD echo %s
 		`, repo)))
-		cli.DockerCmd(c, "push", repo)
+		cli.DockerCmd(t, "push", repo)
 		repos = append(repos, repo)
 	}
 
 	// Clear local images store.
 	args := append([]string{"rmi"}, repos...)
-	cli.DockerCmd(c, args...)
+	cli.DockerCmd(t, args...)
 
 	// Run multiple re-pulls concurrently
 	numPulls := 3
@@ -97,27 +88,19 @@ func testConcurrentPullWholeRepo(c *testing.T) {
 	// package is not goroutine-safe.
 	for i := 0; i != numPulls; i++ {
 		err := <-results
-		assert.NilError(c, err, "concurrent pull failed with error: %v", err)
+		assert.NilError(t, err, "concurrent pull failed with error: %v", err)
 	}
 
 	// Ensure all tags were pulled successfully
 	for _, repo := range repos {
-		cli.DockerCmd(c, "inspect", repo)
-		out := cli.DockerCmd(c, "run", "--rm", repo).Combined()
-		assert.Equal(c, strings.TrimSpace(out), "/bin/sh -c echo "+repo)
+		cli.DockerCmd(t, "inspect", repo)
+		out := cli.DockerCmd(t, "run", "--rm", repo).Combined()
+		assert.Equal(t, strings.TrimSpace(out), "/bin/sh -c echo "+repo)
 	}
 }
 
-func (s *DockerRegistrySuite) TestConcurrentPullWholeRepo(c *testing.T) {
-	testConcurrentPullWholeRepo(c)
-}
-
-func (s *DockerSchema1RegistrySuite) TestConcurrentPullWholeRepo(c *testing.T) {
-	testConcurrentPullWholeRepo(c)
-}
-
-// testConcurrentFailingPull tries a concurrent pull that doesn't succeed.
-func testConcurrentFailingPull(c *testing.T) {
+// TestConcurrentFailingPull tries a concurrent pull that doesn't succeed.
+func (s *DockerRegistrySuite) TestConcurrentFailingPull(t *testing.T) {
 	const imgRepo = privateRegistryURL + "/dockercli/busybox"
 
 	// Run multiple pulls concurrently
@@ -135,40 +118,32 @@ func testConcurrentFailingPull(c *testing.T) {
 	// package is not goroutine-safe.
 	for i := 0; i != numPulls; i++ {
 		err := <-results
-		assert.ErrorContains(c, err, "", "expected pull to fail")
+		assert.ErrorContains(t, err, "", "expected pull to fail")
 	}
 }
 
-func (s *DockerRegistrySuite) TestConcurrentFailingPull(c *testing.T) {
-	testConcurrentFailingPull(c)
-}
-
-func (s *DockerSchema1RegistrySuite) TestConcurrentFailingPull(c *testing.T) {
-	testConcurrentFailingPull(c)
-}
-
-// testConcurrentPullMultipleTags pulls multiple tags from the same repo
+// TestConcurrentPullMultipleTags pulls multiple tags from the same repo
 // concurrently.
-func testConcurrentPullMultipleTags(c *testing.T) {
+func (s *DockerRegistrySuite) TestConcurrentPullMultipleTags(t *testing.T) {
 	const imgRepo = privateRegistryURL + "/dockercli/busybox"
 
 	var repos []string
 	for _, tag := range []string{"recent", "fresh", "todays"} {
 		repo := fmt.Sprintf("%v:%v", imgRepo, tag)
-		buildImageSuccessfully(c, repo, build.WithDockerfile(fmt.Sprintf(`
+		cli.BuildCmd(t, repo, build.WithDockerfile(fmt.Sprintf(`
 		    FROM busybox
 		    ENTRYPOINT ["/bin/echo"]
 		    ENV FOO foo
 		    ENV BAR bar
 		    CMD echo %s
 		`, repo)))
-		cli.DockerCmd(c, "push", repo)
+		cli.DockerCmd(t, "push", repo)
 		repos = append(repos, repo)
 	}
 
 	// Clear local images store.
 	args := append([]string{"rmi"}, repos...)
-	cli.DockerCmd(c, args...)
+	cli.DockerCmd(t, args...)
 
 	// Re-pull individual tags, in parallel
 	results := make(chan error, len(repos))
@@ -184,32 +159,24 @@ func testConcurrentPullMultipleTags(c *testing.T) {
 	// package is not goroutine-safe.
 	for range repos {
 		err := <-results
-		assert.NilError(c, err, "concurrent pull failed with error: %v", err)
+		assert.NilError(t, err, "concurrent pull failed with error: %v", err)
 	}
 
 	// Ensure all tags were pulled successfully
 	for _, repo := range repos {
-		cli.DockerCmd(c, "inspect", repo)
-		out := cli.DockerCmd(c, "run", "--rm", repo).Combined()
-		assert.Equal(c, strings.TrimSpace(out), "/bin/sh -c echo "+repo)
+		cli.DockerCmd(t, "inspect", repo)
+		out := cli.DockerCmd(t, "run", "--rm", repo).Combined()
+		assert.Equal(t, strings.TrimSpace(out), "/bin/sh -c echo "+repo)
 	}
 }
 
-func (s *DockerRegistrySuite) TestConcurrentPullMultipleTags(c *testing.T) {
-	testConcurrentPullMultipleTags(c)
-}
-
-func (s *DockerSchema1RegistrySuite) TestConcurrentPullMultipleTags(c *testing.T) {
-	testConcurrentPullMultipleTags(c)
-}
-
-// testPullIDStability verifies that pushing an image and pulling it back
+// TestPullIDStability verifies that pushing an image and pulling it back
 // preserves the image ID.
-func testPullIDStability(c *testing.T) {
+func (s *DockerRegistrySuite) TestPullIDStability(t *testing.T) {
 	const derivedImage = privateRegistryURL + "/dockercli/id-stability"
 	const baseImage = "busybox"
 
-	buildImageSuccessfully(c, derivedImage, build.WithDockerfile(fmt.Sprintf(`
+	cli.BuildCmd(t, derivedImage, build.WithDockerfile(fmt.Sprintf(`
 	    FROM %s
 	    ENV derived true
 	    ENV asdf true
@@ -217,70 +184,54 @@ func testPullIDStability(c *testing.T) {
 	    CMD echo %s
 	`, baseImage, derivedImage)))
 
-	originalID := getIDByName(c, derivedImage)
-	cli.DockerCmd(c, "push", derivedImage)
+	originalID := getIDByName(t, derivedImage)
+	cli.DockerCmd(t, "push", derivedImage)
 
 	// Pull
-	out := cli.DockerCmd(c, "pull", derivedImage).Combined()
+	out := cli.DockerCmd(t, "pull", derivedImage).Combined()
 	if strings.Contains(out, "Pull complete") {
-		c.Fatalf("repull redownloaded a layer: %s", out)
+		t.Fatalf("repull redownloaded a layer: %s", out)
 	}
 
-	derivedIDAfterPull := getIDByName(c, derivedImage)
+	derivedIDAfterPull := getIDByName(t, derivedImage)
 
 	if derivedIDAfterPull != originalID {
-		c.Fatal("image's ID unexpectedly changed after a repush/repull")
+		t.Fatal("image's ID unexpectedly changed after a repush/repull")
 	}
 
 	// Make sure the image runs correctly
-	out = cli.DockerCmd(c, "run", "--rm", derivedImage).Combined()
+	out = cli.DockerCmd(t, "run", "--rm", derivedImage).Combined()
 	if strings.TrimSpace(out) != derivedImage {
-		c.Fatalf("expected %s; got %s", derivedImage, out)
+		t.Fatalf("expected %s; got %s", derivedImage, out)
 	}
 
 	// Confirm that repushing and repulling does not change the computed ID
-	cli.DockerCmd(c, "push", derivedImage)
-	cli.DockerCmd(c, "rmi", derivedImage)
-	cli.DockerCmd(c, "pull", derivedImage)
+	cli.DockerCmd(t, "push", derivedImage)
+	cli.DockerCmd(t, "rmi", derivedImage)
+	cli.DockerCmd(t, "pull", derivedImage)
 
-	derivedIDAfterPull = getIDByName(c, derivedImage)
+	derivedIDAfterPull = getIDByName(t, derivedImage)
 	if derivedIDAfterPull != originalID {
-		c.Fatal("image's ID unexpectedly changed after a repush/repull")
+		t.Fatal("image's ID unexpectedly changed after a repush/repull")
 	}
 
 	// Make sure the image still runs
-	out = cli.DockerCmd(c, "run", "--rm", derivedImage).Combined()
+	out = cli.DockerCmd(t, "run", "--rm", derivedImage).Combined()
 	if strings.TrimSpace(out) != derivedImage {
-		c.Fatalf("expected %s; got %s", derivedImage, out)
+		t.Fatalf("expected %s; got %s", derivedImage, out)
 	}
 }
 
-func (s *DockerRegistrySuite) TestPullIDStability(c *testing.T) {
-	testPullIDStability(c)
-}
-
-func (s *DockerSchema1RegistrySuite) TestPullIDStability(c *testing.T) {
-	testPullIDStability(c)
-}
-
 // #21213
-func testPullNoLayers(c *testing.T) {
+func (s *DockerRegistrySuite) TestPullNoLayers(t *testing.T) {
 	const imgRepo = privateRegistryURL + "/dockercli/scratch"
 
-	buildImageSuccessfully(c, imgRepo, build.WithDockerfile(`
+	cli.BuildCmd(t, imgRepo, build.WithDockerfile(`
 	FROM scratch
 	ENV foo bar`))
-	cli.DockerCmd(c, "push", imgRepo)
-	cli.DockerCmd(c, "rmi", imgRepo)
-	cli.DockerCmd(c, "pull", imgRepo)
-}
-
-func (s *DockerRegistrySuite) TestPullNoLayers(c *testing.T) {
-	testPullNoLayers(c)
-}
-
-func (s *DockerSchema1RegistrySuite) TestPullNoLayers(c *testing.T) {
-	testPullNoLayers(c)
+	cli.DockerCmd(t, "push", imgRepo)
+	cli.DockerCmd(t, "rmi", imgRepo)
+	cli.DockerCmd(t, "pull", imgRepo)
 }
 
 func (s *DockerRegistrySuite) TestPullManifestList(c *testing.T) {
@@ -289,30 +240,26 @@ func (s *DockerRegistrySuite) TestPullManifestList(c *testing.T) {
 	assert.NilError(c, err, "error setting up image")
 
 	// Inject a manifest list into the registry
-	manifestList := &manifestlist.ManifestList{
-		Versioned: manifest.Versioned{
+	manifestList := &ocispec.Index{
+		Versioned: specs.Versioned{
 			SchemaVersion: 2,
-			MediaType:     manifestlist.MediaTypeManifestList,
 		},
-		Manifests: []manifestlist.ManifestDescriptor{
+		MediaType: ocispec.MediaTypeImageIndex,
+		Manifests: []ocispec.Descriptor{
 			{
-				Descriptor: distribution.Descriptor{
-					Digest:    "sha256:1a9ec845ee94c202b2d5da74a24f0ed2058318bfa9879fa541efaecba272e86b",
-					Size:      3253,
-					MediaType: schema2.MediaTypeManifest,
-				},
-				Platform: manifestlist.PlatformSpec{
+				Digest:    "sha256:1a9ec845ee94c202b2d5da74a24f0ed2058318bfa9879fa541efaecba272e86b",
+				Size:      3253,
+				MediaType: ocispec.MediaTypeImageManifest,
+				Platform: &ocispec.Platform{
 					Architecture: "bogus_arch",
 					OS:           "bogus_os",
 				},
 			},
 			{
-				Descriptor: distribution.Descriptor{
-					Digest:    pushDigest,
-					Size:      3253,
-					MediaType: schema2.MediaTypeManifest,
-				},
-				Platform: manifestlist.PlatformSpec{
+				Digest:    pushDigest,
+				Size:      3253,
+				MediaType: ocispec.MediaTypeImageManifest,
+				Platform: &ocispec.Platform{
 					Architecture: runtime.GOARCH,
 					OS:           runtime.GOOS,
 				},
@@ -455,7 +402,7 @@ func (s *DockerRegistrySuite) TestRunImplicitPullWithNoTag(c *testing.T) {
 	cli.DockerCmd(c, "rmi", repoTag2)
 
 	out := cli.DockerCmd(c, "run", imgRepo).Combined()
-	assert.Assert(c, strings.Contains(out, fmt.Sprintf("Unable to find image '%s:latest' locally", imgRepo)))
+	assert.Assert(c, is.Contains(out, fmt.Sprintf("Unable to find image '%s:latest' locally", imgRepo)))
 	// There should be only one line for repo, the one with repo:latest
 	outImageCmd := cli.DockerCmd(c, "images", imgRepo).Stdout()
 	splitOutImageCmd := strings.Split(strings.TrimSpace(outImageCmd), "\n")
