@@ -1,13 +1,13 @@
-package image // import "github.com/docker/docker/integration/image"
+package image
 
 import (
 	"context"
 	"strings"
 	"testing"
 
-	containertypes "github.com/docker/docker/api/types/container"
-	"github.com/docker/docker/integration/internal/container"
-	"github.com/docker/docker/testutil/daemon"
+	"github.com/moby/moby/client"
+	"github.com/moby/moby/v2/integration/internal/container"
+	"github.com/moby/moby/v2/internal/testutil/daemon"
 	"gotest.tools/v3/assert"
 	is "gotest.tools/v3/assert/cmp"
 	"gotest.tools/v3/poll"
@@ -18,32 +18,32 @@ func TestCommitInheritsEnv(t *testing.T) {
 	skip.If(t, testEnv.DaemonInfo.OSType == "windows", "FIXME")
 	ctx := setupTest(t)
 
-	client := testEnv.APIClient()
+	apiClient := testEnv.APIClient()
 
-	cID1 := container.Create(ctx, t, client)
+	cID1 := container.Create(ctx, t, apiClient)
 	imgName := strings.ToLower(t.Name())
 
-	commitResp1, err := client.ContainerCommit(ctx, cID1, containertypes.CommitOptions{
+	commitResp1, err := apiClient.ContainerCommit(ctx, cID1, client.ContainerCommitOptions{
 		Changes:   []string{"ENV PATH=/bin"},
 		Reference: imgName,
 	})
 	assert.NilError(t, err)
 
-	image1, _, err := client.ImageInspectWithRaw(ctx, commitResp1.ID)
+	image1, err := apiClient.ImageInspect(ctx, commitResp1.ID)
 	assert.NilError(t, err)
 
 	expectedEnv1 := []string{"PATH=/bin"}
 	assert.Check(t, is.DeepEqual(expectedEnv1, image1.Config.Env))
 
-	cID2 := container.Create(ctx, t, client, container.WithImage(image1.ID))
+	cID2 := container.Create(ctx, t, apiClient, container.WithImage(image1.ID))
 
-	commitResp2, err := client.ContainerCommit(ctx, cID2, containertypes.CommitOptions{
+	commitResp2, err := apiClient.ContainerCommit(ctx, cID2, client.ContainerCommitOptions{
 		Changes:   []string{"ENV PATH=/usr/bin:$PATH"},
 		Reference: imgName,
 	})
 	assert.NilError(t, err)
 
-	image2, _, err := client.ImageInspectWithRaw(ctx, commitResp2.ID)
+	image2, err := apiClient.ImageInspect(ctx, commitResp2.ID)
 	assert.NilError(t, err)
 	expectedEnv2 := []string{"PATH=/usr/bin:/bin"}
 	assert.Check(t, is.DeepEqual(expectedEnv2, image2.Config.Env))
@@ -64,7 +64,7 @@ func TestUsernsCommit(t *testing.T) {
 
 	cID := container.Run(ctx, t, clientUserRemap, container.WithName(t.Name()), container.WithImage("busybox"), container.WithCmd("sh", "-c", "echo hello world > /hello.txt && chown 1000:1000 /hello.txt"))
 	poll.WaitOn(t, container.IsStopped(ctx, clientUserRemap, cID))
-	img, err := clientUserRemap.ContainerCommit(ctx, t.Name(), containertypes.CommitOptions{})
+	img, err := clientUserRemap.ContainerCommit(ctx, t.Name(), client.ContainerCommitOptions{})
 	assert.NilError(t, err)
 
 	res := container.RunAttach(ctx, t, clientUserRemap, container.WithImage(img.ID), container.WithCmd("sh", "-c", "stat -c %u:%g /hello.txt"))
