@@ -1,4 +1,4 @@
-package image // import "github.com/docker/docker/integration/image"
+package image
 
 import (
 	"strings"
@@ -7,10 +7,14 @@ import (
 	cerrdefs "github.com/containerd/errdefs"
 	"github.com/containerd/platforms"
 
-	containertypes "github.com/docker/docker/api/types/container"
-	"github.com/docker/docker/api/types/image"
-	"github.com/docker/docker/integration/internal/container"
-	"github.com/docker/docker/internal/testutils/specialimage"
+	"github.com/moby/moby/api/types/image"
+	"github.com/moby/moby/client"
+	"github.com/moby/moby/client/pkg/stringid"
+	build "github.com/moby/moby/v2/integration/internal/build"
+	"github.com/moby/moby/v2/integration/internal/container"
+	iimage "github.com/moby/moby/v2/integration/internal/image"
+	"github.com/moby/moby/v2/internal/testutil/fakecontext"
+	"github.com/moby/moby/v2/internal/testutil/specialimage"
 	ocispec "github.com/opencontainers/image-spec/specs-go/v1"
 	"gotest.tools/v3/assert"
 	is "gotest.tools/v3/assert/cmp"
@@ -20,47 +24,47 @@ import (
 func TestRemoveImageOrphaning(t *testing.T) {
 	ctx := setupTest(t)
 
-	client := testEnv.APIClient()
+	apiClient := testEnv.APIClient()
 
 	imgName := strings.ToLower(t.Name())
 
 	// Create a container from busybox, and commit a small change so we have a new image
-	cID1 := container.Create(ctx, t, client, container.WithCmd(""))
-	commitResp1, err := client.ContainerCommit(ctx, cID1, containertypes.CommitOptions{
+	cID1 := container.Create(ctx, t, apiClient, container.WithCmd(""))
+	commitResp1, err := apiClient.ContainerCommit(ctx, cID1, client.ContainerCommitOptions{
 		Changes:   []string{`ENTRYPOINT ["true"]`},
 		Reference: imgName,
 	})
 	assert.NilError(t, err)
 
 	// verifies that reference now points to first image
-	resp, err := client.ImageInspect(ctx, imgName)
+	resp, err := apiClient.ImageInspect(ctx, imgName)
 	assert.NilError(t, err)
 	assert.Check(t, is.Equal(resp.ID, commitResp1.ID))
 
 	// Create a container from created image, and commit a small change with same reference name
-	cID2 := container.Create(ctx, t, client, container.WithImage(imgName), container.WithCmd(""))
-	commitResp2, err := client.ContainerCommit(ctx, cID2, containertypes.CommitOptions{
+	cID2 := container.Create(ctx, t, apiClient, container.WithImage(imgName), container.WithCmd(""))
+	commitResp2, err := apiClient.ContainerCommit(ctx, cID2, client.ContainerCommitOptions{
 		Changes:   []string{`LABEL Maintainer="Integration Tests"`},
 		Reference: imgName,
 	})
 	assert.NilError(t, err)
 
 	// verifies that reference now points to second image
-	resp, err = client.ImageInspect(ctx, imgName)
+	resp, err = apiClient.ImageInspect(ctx, imgName)
 	assert.NilError(t, err)
 	assert.Check(t, is.Equal(resp.ID, commitResp2.ID))
 
 	// try to remove the image, should not error out.
-	_, err = client.ImageRemove(ctx, imgName, image.RemoveOptions{})
+	_, err = apiClient.ImageRemove(ctx, imgName, client.ImageRemoveOptions{})
 	assert.NilError(t, err)
 
 	// check if the first image is still there
-	resp, err = client.ImageInspect(ctx, commitResp1.ID)
+	resp, err = apiClient.ImageInspect(ctx, commitResp1.ID)
 	assert.NilError(t, err)
 	assert.Check(t, is.Equal(resp.ID, commitResp1.ID))
 
 	// check if the second image has been deleted
-	_, err = client.ImageInspect(ctx, commitResp2.ID)
+	_, err = apiClient.ImageInspect(ctx, commitResp2.ID)
 	assert.Check(t, is.ErrorContains(err, "No such image:"))
 }
 
@@ -68,12 +72,12 @@ func TestRemoveByDigest(t *testing.T) {
 	skip.If(t, !testEnv.UsingSnapshotter(), "RepoDigests doesn't include tags when using graphdrivers")
 
 	ctx := setupTest(t)
-	client := testEnv.APIClient()
+	apiClient := testEnv.APIClient()
 
-	err := client.ImageTag(ctx, "busybox", "test-remove-by-digest:latest")
+	_, err := apiClient.ImageTag(ctx, client.ImageTagOptions{Source: "busybox", Target: "test-remove-by-digest:latest"})
 	assert.NilError(t, err)
 
-	inspect, err := client.ImageInspect(ctx, "test-remove-by-digest")
+	inspect, err := apiClient.ImageInspect(ctx, "test-remove-by-digest")
 	assert.NilError(t, err)
 
 	id := ""
@@ -85,15 +89,15 @@ func TestRemoveByDigest(t *testing.T) {
 	}
 	assert.Assert(t, id != "")
 
-	_, err = client.ImageRemove(ctx, id, image.RemoveOptions{})
+	_, err = apiClient.ImageRemove(ctx, id, client.ImageRemoveOptions{})
 	assert.NilError(t, err, "error removing %s", id)
 
-	_, err = client.ImageInspect(ctx, "busybox")
+	_, err = apiClient.ImageInspect(ctx, "busybox")
 	assert.NilError(t, err, "busybox image got deleted")
 
-	inspect, err = client.ImageInspect(ctx, "test-remove-by-digest")
+	inspect, err = apiClient.ImageInspect(ctx, "test-remove-by-digest")
 	assert.Check(t, is.ErrorType(err, cerrdefs.IsNotFound))
-	assert.Check(t, is.DeepEqual(inspect, image.InspectResponse{}))
+	assert.Check(t, is.DeepEqual(inspect, client.ImageInspectResult{}))
 }
 
 func TestRemoveWithPlatform(t *testing.T) {
@@ -115,7 +119,7 @@ func TestRemoveWithPlatform(t *testing.T) {
 
 	var imageIdx *ocispec.Index
 	var descs []ocispec.Descriptor
-	specialimage.Load(ctx, t, apiClient, func(dir string) (*ocispec.Index, error) {
+	iimage.Load(ctx, t, apiClient, func(dir string) (*ocispec.Index, error) {
 		idx, d, err := specialimage.MultiPlatform(dir, imgName, []ocispec.Platform{
 			platformHost,
 			{
@@ -138,29 +142,29 @@ func TestRemoveWithPlatform(t *testing.T) {
 		platform *ocispec.Platform
 		deleted  ocispec.Descriptor
 	}{
-		{&platformHost, descs[0]},
-		{&someOtherPlatform, descs[3]},
+		{platform: &platformHost, deleted: descs[0]},
+		{platform: &someOtherPlatform, deleted: descs[3]},
 	} {
-		resp, err := apiClient.ImageRemove(ctx, imgName, image.RemoveOptions{
+		res, err := apiClient.ImageRemove(ctx, imgName, client.ImageRemoveOptions{
 			Platforms: []ocispec.Platform{*tc.platform},
 			Force:     true,
 		})
 		assert.NilError(t, err)
-		assert.Check(t, is.Len(resp, 1))
-		for _, r := range resp {
+		assert.Check(t, is.Len(res.Items, 1))
+		for _, r := range res.Items {
 			assert.Check(t, is.Equal(r.Untagged, ""), "No image should be untagged")
 		}
-		checkPlatformDeleted(t, imageIdx, resp, tc.deleted)
+		checkPlatformDeleted(t, imageIdx, res.Items, tc.deleted)
 	}
 
 	// Delete the rest
-	resp, err := apiClient.ImageRemove(ctx, imgName, image.RemoveOptions{})
+	resp, err := apiClient.ImageRemove(ctx, imgName, client.ImageRemoveOptions{})
 	assert.NilError(t, err)
 
-	assert.Check(t, is.Len(resp, 2))
-	assert.Check(t, is.Equal(resp[0].Untagged, imgName))
-	assert.Check(t, is.Equal(resp[1].Deleted, imageIdx.Manifests[0].Digest.String()))
-	// TODO: Should it also include platform-specific manifests?
+	assert.Check(t, is.Len(resp.Items, 2))
+	assert.Check(t, is.Equal(resp.Items[0].Untagged, imgName))
+	assert.Check(t, is.Equal(resp.Items[1].Deleted, imageIdx.Manifests[0].Digest.String()))
+	// TODO(vvoland): Should it also include platform-specific manifests? https://github.com/moby/moby/pull/49982
 }
 
 func checkPlatformDeleted(t *testing.T, imageIdx *ocispec.Index, resp []image.DeleteResponse, mfstDesc ocispec.Descriptor) {
@@ -175,3 +179,39 @@ func checkPlatformDeleted(t *testing.T, imageIdx *ocispec.Index, resp []image.De
 		}
 	}
 }
+
+func TestAPIImagesDelete(t *testing.T) {
+	ctx := setupTest(t)
+	apiClient := testEnv.APIClient()
+
+	const name = "test-api-images-delete"
+
+	buildCtx := fakecontext.New(t, t.TempDir(),
+		fakecontext.WithDockerfile(`FROM busybox
+ENV FOO=bar`))
+	defer buildCtx.Close()
+
+	imgID := build.Do(ctx, t, apiClient, buildCtx)
+
+	// Cleanup always runs
+	defer func() {
+		_, _ = apiClient.ImageRemove(ctx, imgID, client.ImageRemoveOptions{Force: true})
+	}()
+
+	_, err := apiClient.ImageTag(ctx, client.ImageTagOptions{Source: imgID, Target: name})
+	assert.NilError(t, err)
+
+	_, err = apiClient.ImageTag(ctx, client.ImageTagOptions{Source: imgID, Target: "test:tag1"})
+	assert.NilError(t, err)
+
+	_, err = apiClient.ImageRemove(ctx, imgID, client.ImageRemoveOptions{})
+	assert.Check(t, is.ErrorType(err, cerrdefs.IsConflict))
+	assert.Check(t, is.ErrorContains(err, "unable to delete "+stringid.TruncateID(imgID)))
+
+	_, err = apiClient.ImageRemove(ctx, "test:noexist", client.ImageRemoveOptions{})
+	assert.Check(t, is.ErrorType(err, cerrdefs.IsNotFound))
+	assert.Check(t, is.ErrorContains(err, "No such image: test:noexist"))
+
+	_, err = apiClient.ImageRemove(ctx, "test:tag1", client.ImageRemoveOptions{})
+	assert.NilError(t, err)
+}
