@@ -4,26 +4,23 @@ import (
 	"context"
 	"fmt"
 
-	"github.com/docker/docker/api/types/filters"
-	swarmtypes "github.com/docker/docker/api/types/swarm"
-	"github.com/docker/docker/client"
+	swarmtypes "github.com/moby/moby/api/types/swarm"
+	"github.com/moby/moby/client"
 	"gotest.tools/v3/poll"
 )
 
 // NoTasksForService verifies that there are no more tasks for the given service
-func NoTasksForService(ctx context.Context, client client.ServiceAPIClient, serviceID string) func(log poll.LogT) poll.Result {
+func NoTasksForService(ctx context.Context, apiClient client.TaskAPIClient, serviceID string) func(log poll.LogT) poll.Result {
 	return func(log poll.LogT) poll.Result {
-		tasks, err := client.TaskList(ctx, swarmtypes.TaskListOptions{
-			Filters: filters.NewArgs(
-				filters.Arg("service", serviceID),
-			),
+		taskList, err := apiClient.TaskList(ctx, client.TaskListOptions{
+			Filters: make(client.Filters).Add("service", serviceID),
 		})
 		if err == nil {
-			if len(tasks) == 0 {
+			if len(taskList.Items) == 0 {
 				return poll.Success()
 			}
-			if len(tasks) > 0 {
-				return poll.Continue("task count for service %s at %d waiting for 0", serviceID, len(tasks))
+			if len(taskList.Items) > 0 {
+				return poll.Continue("task count for service %s at %d waiting for 0", serviceID, len(taskList.Items))
 			}
 			return poll.Continue("waiting for tasks for service %s to be deleted", serviceID)
 		}
@@ -33,36 +30,35 @@ func NoTasksForService(ctx context.Context, client client.ServiceAPIClient, serv
 }
 
 // NoTasks verifies that all tasks are gone
-func NoTasks(ctx context.Context, client client.ServiceAPIClient) func(log poll.LogT) poll.Result {
+func NoTasks(ctx context.Context, apiClient client.TaskAPIClient) func(log poll.LogT) poll.Result {
 	return func(log poll.LogT) poll.Result {
-		tasks, err := client.TaskList(ctx, swarmtypes.TaskListOptions{})
+		taskResult, err := apiClient.TaskList(ctx, client.TaskListOptions{})
 		switch {
 		case err != nil:
 			return poll.Error(err)
-		case len(tasks) == 0:
+		case len(taskResult.Items) == 0:
 			return poll.Success()
 		default:
-			return poll.Continue("waiting for all tasks to be removed: task count at %d", len(tasks))
+			return poll.Continue("waiting for all tasks to be removed: task count at %d", len(taskResult.Items))
 		}
 	}
 }
 
 // RunningTasksCount verifies there are `instances` tasks running for `serviceID`
-func RunningTasksCount(ctx context.Context, client client.ServiceAPIClient, serviceID string, instances uint64) func(log poll.LogT) poll.Result {
+func RunningTasksCount(ctx context.Context, apiClient client.TaskAPIClient, serviceID string, instances uint64) func(log poll.LogT) poll.Result {
 	return func(log poll.LogT) poll.Result {
-		filter := filters.NewArgs()
-		filter.Add("service", serviceID)
-		tasks, err := client.TaskList(ctx, swarmtypes.TaskListOptions{
-			Filters: filter,
+		taskList, err := apiClient.TaskList(ctx, client.TaskListOptions{
+			Filters: make(client.Filters).Add("service", serviceID),
 		})
 		var running int
 		var taskError string
-		for _, task := range tasks {
+		for _, task := range taskList.Items {
 			switch task.Status.State {
 			case swarmtypes.TaskStateRunning:
 				running++
-			case swarmtypes.TaskStateFailed:
+			case swarmtypes.TaskStateFailed, swarmtypes.TaskStateRejected:
 				if task.Status.Err != "" {
+					log.Logf("task %v on node %v %v: %v", task.ID, task.NodeID, task.Status.State, task.Status.Err)
 					taskError = task.Status.Err
 				}
 			default:
@@ -80,7 +76,7 @@ func RunningTasksCount(ctx context.Context, client client.ServiceAPIClient, serv
 		case running == int(instances):
 			return poll.Success()
 		default:
-			return poll.Continue("running task count at %d waiting for %d (total tasks: %d)", running, instances, len(tasks))
+			return poll.Continue("running task count at %d waiting for %d (total tasks: %d)", running, instances, len(taskList.Items))
 		}
 	}
 }
@@ -88,8 +84,8 @@ func RunningTasksCount(ctx context.Context, client client.ServiceAPIClient, serv
 // JobComplete is a poll function for determining that a ReplicatedJob is
 // completed additionally, while polling, it verifies that the job never
 // exceeds MaxConcurrent running tasks
-func JobComplete(ctx context.Context, client client.ServiceAPIClient, service swarmtypes.Service) func(log poll.LogT) poll.Result {
-	filter := filters.NewArgs(filters.Arg("service", service.ID))
+func JobComplete(ctx context.Context, apiClient client.TaskAPIClient, service swarmtypes.Service) func(log poll.LogT) poll.Result {
+	filter := make(client.Filters).Add("service", service.ID)
 
 	var jobIteration swarmtypes.Version
 	if service.JobStatus != nil {
@@ -101,7 +97,7 @@ func JobComplete(ctx context.Context, client client.ServiceAPIClient, service sw
 	previousResult := ""
 
 	return func(log poll.LogT) poll.Result {
-		tasks, err := client.TaskList(ctx, swarmtypes.TaskListOptions{
+		taskList, err := apiClient.TaskList(ctx, client.TaskListOptions{
 			Filters: filter,
 		})
 		if err != nil {
@@ -114,7 +110,7 @@ func JobComplete(ctx context.Context, client client.ServiceAPIClient, service sw
 		var runningSlot []int
 		var runningID []string
 
-		for _, task := range tasks {
+		for _, task := range taskList.Items {
 			// make sure the task has the same job iteration
 			if task.JobIteration == nil || task.JobIteration.Index != jobIteration.Index {
 				continue
@@ -160,3 +156,20 @@ func JobComplete(ctx context.Context, client client.ServiceAPIClient, service sw
 		}
 	}
 }
+
+func HasLeader(ctx context.Context, apiClient client.NodeAPIClient) func(log poll.LogT) poll.Result {
+	return func(log poll.LogT) poll.Result {
+		result, err := apiClient.NodeList(ctx, client.NodeListOptions{
+			Filters: make(client.Filters).Add("role", "manager"),
+		})
+		if err != nil {
+			return poll.Error(err)
+		}
+		for _, node := range result.Items {
+			if node.ManagerStatus != nil && node.ManagerStatus.Leader {
+				return poll.Success()
+			}
+		}
+		return poll.Continue("no leader elected yet")
+	}
+}
