@@ -1,25 +1,39 @@
 package networking
 
 import (
+	"os"
 	"testing"
 
-	"github.com/docker/docker/client"
-	"github.com/docker/docker/internal/testutils/networking"
-	"github.com/docker/docker/testutil/request"
+	"github.com/moby/moby/client"
+	"github.com/moby/moby/v2/integration/internal/testutils/networking"
+	"github.com/moby/moby/v2/internal/testutil/daemon"
+	"github.com/moby/moby/v2/internal/testutil/request"
 	"gotest.tools/v3/assert"
 	is "gotest.tools/v3/assert/cmp"
 )
 
+const defaultFirewallBackend = "iptables"
+
 func TestInfoFirewallBackend(t *testing.T) {
 	ctx := setupTest(t)
-	c := testEnv.APIClient()
 
-	expDriver := "iptables"
-	if !testEnv.IsRootless() && networking.FirewalldRunning() {
-		expDriver = "iptables+firewalld"
+	d := daemon.New(t)
+	d.StartWithBusybox(ctx, t)
+	t.Cleanup(func() { d.Stop(t) })
+
+	c := d.NewClientT(t)
+	t.Cleanup(func() { c.Close() })
+
+	expDriver := defaultFirewallBackend
+	if val := os.Getenv("DOCKER_FIREWALL_BACKEND"); val != "" {
+		expDriver = val
 	}
-	info, err := c.Info(ctx)
+	if !testEnv.IsRootless() && networking.FirewalldRunning() {
+		expDriver += "+firewalld"
+	}
+	result, err := c.Info(ctx, client.InfoOptions{})
 	assert.NilError(t, err)
+	info := result.Info
 	assert.Assert(t, info.FirewallBackend != nil, "expected firewall backend in info response")
 	t.Log("FirewallBackend: Driver:", info.FirewallBackend.Driver)
 	for _, kv := range info.FirewallBackend.Info {
@@ -29,9 +43,10 @@ func TestInfoFirewallBackend(t *testing.T) {
 
 	// Check FirewallBackend is omitted for API <= 1.48.
 	t.Run("api 1.48", func(t *testing.T) {
-		c148 := request.NewAPIClient(t, client.WithVersion("1.48"))
-		info148, err := c148.Info(ctx)
+		c148 := request.NewAPIClient(t, client.WithAPIVersion("1.48"))
+		result, err := c148.Info(ctx, client.InfoOptions{})
 		assert.NilError(t, err)
+		info148 := result.Info
 		assert.Check(t, is.Nil(info148.FirewallBackend))
 	})
 }
