@@ -1,8 +1,18 @@
-// FIXME(thaJeztah): remove once we are a module; the go:build directive prevents go from downgrading language version to go1.16:
-//go:build go1.23
-
 package sliceutil
 
+func Deref[T any](slice []*T) []T {
+	if slice == nil {
+		return nil
+	}
+	out := make([]T, 0, len(slice))
+	for _, p := range slice {
+		if p != nil {
+			out = append(out, *p)
+		}
+	}
+	return out
+}
+
 func Dedup[T comparable](slice []T) []T {
 	keys := make(map[T]struct{})
 	out := make([]T, 0, len(slice))
@@ -16,6 +26,9 @@ func Dedup[T comparable](slice []T) []T {
 }
 
 func Map[S ~[]In, In, Out any](s S, fn func(In) Out) []Out {
+	if s == nil {
+		return nil
+	}
 	res := make([]Out, len(s))
 	for i, v := range s {
 		res[i] = fn(v)
@@ -25,10 +38,6 @@ func Map[S ~[]In, In, Out any](s S, fn func(In) Out) []Out {
 
 func Mapper[In, Out any](fn func(In) Out) func([]In) []Out {
 	return func(s []In) []Out {
-		res := make([]Out, len(s))
-		for i, v := range s {
-			res[i] = fn(v)
-		}
-		return res
+		return Map(s, fn)
 	}
 }
